;-----------------------------------------------------------------------------
; Quickie Procedure to set up the Global Variables:
;  NKey = 'number of key (unique) fields selected'
;  NDat = 'number of Data fields to DeNormalize'
;  Keyfld[] = 'array holding the field names of the key fields selected'
;  Datfld[] = 'array holding the field names of the data fields selected'
;  Col.num = 'Starting column number for data in denormalized table
;  Answer Table -- generated by checking all key and data fields
;
; This proc calls up a table (Tabl) in Queryview.  It will let the user
; checkmark <F6> the key (or unique) fields.  When <F2> is pressed, then it
; will allow the user to checkmark the data fields to denormalize.  If <Esc>
; is pressed it will return false.  If the user presses <F2> it will return
; true and the Global variables above.
;-----------------------------------------------------------------------------
Proc InitDeNorm(Tabl)
Private TKey, TDat, I, Prmpt
 I = NFields(Tabl)
 Array TKey[I]
 Array TDat[I]
 NKey = 0
 NDat = 0
 Menu {Ask} Select Tabl
 While True
  Wait Record
   Prompt "<F6>-Choose Key (Unique) fields,  <F2>-Continue"
   Until "Esc","F2","F6"
   Switch
    Case Retval = "Esc":
     Return False
    Case Retval = "F2":
     While True
      Wait Record
       Prompt "<F6>-Choose Data to Denormalize, <F2>-Process Request"
       Until "Esc","F2","F6"
       Switch
        Case Retval = "Esc":
         Return False
        Case Retval = "F2":
         Do_it!
         Array Keyfld[Nkey]
         Array Datfld[NDat]
         For I from 1 to Nkey
          Keyfld[I] = TKey[I]
         Endfor
         For I from 1 to NDat
          Datfld[I] = TDat[I]
         Endfor
         Rename "Answer" "Temp1"
         Col.num = 1
         Return True
        Case Retval = "F6" and Checkmarkstatus() = "" and Field() <> "#":
         Check
         NDat = NDat + 1
         TDat[NDat] = Field()
       Endswitch
      Endwhile
    Case Retval = "F6" and Checkmarkstatus() = "" and Field() <> "#":
     Check
     NKey = NKey + 1
     TKey[NKey] = Field()
   Endswitch
 Endwhile
Endproc

;--------------------------------------------------------------------------
; This proc places example elements on all the key fields
; Op = "" or = "!".  'Op' is added onto the end of the example elements and
; is used here to place the inclusion operator (!)
;---------------------------------------------------------------------------
Proc LinkKey(Op)
Private I
 For I from 1 to Nkey
  Moveto Field Keyfld[I]
  [] = "_Key"+Strval(I)+Op
 Endfor
Endproc

;---------------------------------------------------------------------------
; This proc places example elements on data fields BDat through EDat.
; If it is the Answer table, Example elements will be placed on the set of
; Data columns designated by the Col.num global variable.
; Op = "" or = "!".  'Op' is added onto the end of the example elements and
; is used here to place the inclusion operator (!).
;---------------------------------------------------------------------------
Proc LinkDat(BDat,EDat,Op)
Private I, ColAdd
 ColAdd = ""
 If Table() = "Answer" then
  ColAdd = "-"+Strval(Col.num)
 Endif
 For I from BDat to EDat
  Moveto Field Datfld[I]+ColAdd
  [] = "_Dat"+Strval(I)+Op
 Endfor
EndProc

;---------------------------------------------------------------------------
; DeNorm takes the answer table (if Link = True) and links it to the "Temp1"
; table, which contains the records from the original Table that are yet to
; be Denormalized.  It will then step through the datafields one by one
; pulling out the Minimum value of each field respectively.  These
; fields are added one by one to the current Col.num in the answer table
;---------------------------------------------------------------------------
Proc DeNorm(Link)                       ; Link = True  Links answer table
Private I,J
 For I from 1 to NDat                   ;Step through the Data fields
  ClearAll
  If Link then                         ; If the answer table is to be linked
   Menu {Ask} Select "Answer" Check     ; Query Answer Table
   LinkKey("!")                         ;  Example (!) elements on Key fields
   LinkDat(1,I-1,"!")                   ;  and Data fields already processed
  Endif
  Menu {Ask} Select "Temp1"            ; Query Temp1 Table
  If Link then                         ; If the answer table is linked
   LinkKey("")                          ;  Example elements on Key fields
   LinkDat(1,I-1,"")                    ;  and Data fields
    Else                                ;    Else
   For J from 1 to NKey                 ; First time through (Link=False)
    Moveto Field Keyfld[J]              ; Set up the initial answer table
    Check
   Endfor
   Link = True
  Endif
  Moveto Field Datfld[I]
  [] = "Calc Min as "+Datfld[I]+"-"+Strval(Col.num)
  Do_it!                                ; Calc the minimum values for the
 Endfor                                 ; Data field being processed
Endproc                                 ; and add it to the answer table

;---------------------------------------------------------------------------
; Subtr deletes (subtracts) the records in the Temp1 table which correspond
; to the records added to the denormalized Answer table columns.  It uses
; the Col.no global variable to determine the set of data to delete.
;---------------------------------------------------------------------------
Proc Subtr()
 ClearAll
 Menu {Ask} Select "Answer"             ; Query the answer table
 LinkKey("")                            ;  Example elmts on all key flds
 LinkDat(1,NDat,"")                     ;  Example elmts on all Dat flds
 Menu {Ask} Select "Temp1"              ; Delete Query the Temp1 table
 Typein "Delete"                        ;  Take out all records
 LinkKey("")                            ;  added to the answer tbl
 LinkDat(1,NDat,"")                     ;  for last Col.num
 Do_it!
Endproc

;----------------------------
; Main Procedure Demo
;----------------------------
If Not InitDeNorm("Demo") then         ; Set up global variables
 Return
Endif
DeNorm(False)                          ; First time through
Subtr()                                ;  Denorm Col.num = 1
While Not IsEmpty("Temp1")           ; While there's still more to DeNorm
 Col.num = Col.num + 1                 ;  Increment Col.num
 DeNorm(True)                          ;  Denorm new data columns
 Subtr()                               ;  Delete them from Temp1
Endwhile
Reset                                  ; View the result
View "Answer"
